/* $RCSfile: main.cxx,v $
 * $Revision: 1.4 $ $Date: 2006/08/31 19:45:37 $
 * Auth: David Loffredo (loffredo@steptools.com)
 * 
 * A Sample Program that demonstrates the STEPNCWrite class.  This
 * creates a file foobar.stp containing a toolpath that simply wiggles
 * around a bit using 3-axis and 5-axis moves.
 * 
 */

#include "STEPNCWrite.h"

int main (int argc, char ** argv)
{
    STEPNCWrite nc;

    printf ("Writing sample file to foobar.238\n");

    nc.Open ("foobar.238", "a sample file", "MonkeyCad");
    nc.Inches();

    nc.Cutter(1,0,0,0,0,0,2);
    nc.LoadTool(1);

    nc.Feedrate (200);
    nc.SpindleSpeed (1000);
    nc.CoolantOn();

    // Do some 5-axis moves
    nc.GoToXYZ ("first point", 0,0,0);
    nc.GoToXYZ_IJK ("", 1,2,3, 0,1,0);
    nc.GoToXYZ_IJK ("", 1,4,3, 0,1,0);

    // Go back to 3-axis moves
    nc.GoToXYZ ("", 10,0,0);

    // Do an ARC 
    nc.Arc ("", -10,0,0, 0,0,0, 10, 1);

    // Do an arc in a different plane
    nc.GoToXYZ_IJK ("", -10,0,10, 0,1,0);
    
    // Do an ARC 
    nc.SetDirection (0,1,0, -1,0,0);
    nc.Arc ("", -10,0,-10, -10,0,0, 10, 0);

    // Go back to 3-axis moves
    nc.GoToXYZ ("", 10,20,30);

    // Do some rapid 3-axis moves
    nc.Rapid();

    int i;
    for (i=5; i<20; i+=5) {
	nc.GoToXYZ ("", 10.1+i,20.2+i,30.3+i);
    }

    // Exercise some of the other NC functions
    nc.PPrint ("", "Hello World");
    nc.GoHome ("");
    nc.Stop ("");

    nc.Close();

    return 0;
}
